// importation des classes contenues dans le package mesh
// (dont notamment Delaunay, Voronoi…)
import megamu.mesh.*;

// déclarer un array d'objets de type Noeud
Noeud[] noeuds;

// déclarer un objet de type Delaunay
// c'est-à-dire qu'on dit que ça existe
Delaunay maillage;

// déclarer ma variable d'image
PImage image;

void setup() {  
  // charger mon fichier d'image source
  image = loadImage("donald.png");
  image.loadPixels();

  size(image.width*10, image.height*10);

  // affecter les valeurs de couleurs à notre tableau de noeuds : on le remplit !
  noeuds = new Noeud[image.width*image.height];
  for (int i = 0; i < image.pixels.length; i ++) {
    noeuds[i] = new Noeud();
    noeuds[i].x = (i%image.width)+random(-1, 1);
    noeuds[i].y = int(i/image.width)+random(-1, 1);
    noeuds[i].couleurDeRemplissage = color(image.pixels[i]);
    noeuds[i].couleurDeContour = 255;
    noeuds[i].rayon = 5;
  }
}

void updateMaillage(){
  // on doit passer par un array multi-dim pour Delaunay
  float[][] positions = new float[noeuds.length][2];
  for (int i=0; i<noeuds.length; i++) {
    positions[i][0] = noeuds[i].x;
    positions[i][1] = noeuds[i].y;
  }

  // mettre à jour mon calcul de maillage
  maillage = new Delaunay(positions);
}

void draw() {
  background(255);

  // faire évoluer la position d'index 0 selon la position
  // de la souris
  noeuds[0].x = mouseX/10.0;
  noeuds[0].y = mouseY/10.0;

  updateMaillage();

  int[] linksAuNoeudZero = maillage.getLinked(0);
  for (int i=0; i<linksAuNoeudZero.length; i++) {
    noeuds[linksAuNoeudZero[i]].x += random(-2, 2);
    noeuds[linksAuNoeudZero[i]].y += random(-2, 2);
  }

  updateMaillage();

  // dessiner les liens entre les points les plus proches
  // pour faire apparaître la triangulation
  float[][] segments = maillage.getEdges();
  for (int i=0; i<segments.length; i++) {
    stroke(0);
    line(segments[i][0]*10, segments[i][1]*10, segments[i][2]*10, segments[i][3]*10);
  }

  // dessiner mes noeuds en passant en revue chacune des lignes
  // de mon tableau de noeuds
  for (int i=0; i<noeuds.length; i++) {
    fill(noeuds[i].couleurDeRemplissage);
    noStroke(); //(noeuds[i].couleurDeContour);
    ellipse(noeuds[i].x*10, noeuds[i].y*10, noeuds[i].rayon, noeuds[i].rayon);
  }
}

