// importation des classes contenues dans le package mesh
// (dont notamment Delaunay, Voronoi…)
import megamu.mesh.*;

// déclarer un array d'objets de type Noeud
Noeud[] noeuds;

// déclarer un objet de type Delaunay
// c'est-à-dire qu'on dit que ça existe
Delaunay maillage;

// déclarer ma variable d'image
PImage image;

void setup() {  
  // charger mon fichier d'image source
  image = loadImage("donald_.jpg");
  image.loadPixels();
  
  size(image.width,image.height);

  // affecter les valeurs de couleurs à notre tableau de noeuds : on le remplit !
  noeuds = new Noeud[2000];
  for (int i=0; i<noeuds.length; i++){
    int v = int(random(image.pixels.length));
    noeuds[i] = new Noeud();
    noeuds[i].x = (v%image.width);
    noeuds[i].y = int(v/image.width);
    noeuds[i].couleurDeRemplissage = color(image.pixels[v]);
    noeuds[i].couleurDeContour = 255;
    noeuds[i].rayon = 5;
  }
}

void draw() {
  background(255);
  
  // faire évoluer la position d'index 0 selon la position
  // de la souris
  noeuds[0].x = mouseX;
  noeuds[0].y = mouseY;
  
  // on doit passer par un array multi-dim pour Delaunay
  float[][] positions = new float[noeuds.length][2];
  for (int i=0; i<noeuds.length; i++){
    positions[i][0] = noeuds[i].x;
    positions[i][1] = noeuds[i].y;
  }
  
  // mettre à jour mon calcul de maillage
  maillage = new Delaunay(positions);
  
  // dessiner les liens entre les points les plus proches
  // pour faire apparaître la triangulation
  float[][] segments = maillage.getEdges();
  for (int i=0; i<segments.length; i++){
    stroke(0);
    line(segments[i][0],segments[i][1],segments[i][2],segments[i][3]);
  }
  
  // dessiner mes noeuds en passant en revue chacune des lignes
  // de mon tableau de noeuds
  for (int i=0; i<noeuds.length; i++) {
    fill(noeuds[i].couleurDeRemplissage);
    noStroke(); //(noeuds[i].couleurDeContour);
    ellipse(noeuds[i].x,noeuds[i].y,noeuds[i].rayon,noeuds[i].rayon); 
  }
}

